package superuserPOC.server;

import com.intellij.openapi.util.text.StringUtil;
import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import jetbrains.buildServer.controllers.interceptors.auth.HttpAuthenticationResult;
import jetbrains.buildServer.controllers.interceptors.auth.HttpAuthenticationScheme;
import jetbrains.buildServer.controllers.interceptors.auth.util.HttpAuthUtil;
import jetbrains.buildServer.serverSide.auth.ServerPrincipal;
import jetbrains.buildServer.users.UserModel;
import org.jetbrains.annotations.NotNull;

/**
 * @author Yegor.Yarko
 *         Date: 19.11.12
 *  WARNING: This API is not TeamCity open API, will change in TeamCity 8.0
 */
public class SuperUserHttpAuthenticationScheme implements HttpAuthenticationScheme {
  private SuperUserData mySuperUserData;
  private UserModel myUserModel;

  public SuperUserHttpAuthenticationScheme(SuperUserData superUserData, UserModel userModel) {
    mySuperUserData = superUserData;
    myUserModel = userModel;
  }

  @NotNull
  public HttpAuthenticationResult processAuthenticationRequest(@NotNull final HttpServletRequest request,
                                                               @NotNull final HttpServletResponse response) throws IOException {
    // here goes the authentication logic
    final String username = request.getParameter(SuperUserData.USERNAME_PARAMETER_NAME);
    final String token = request.getParameter(SuperUserData.TOKEN_PARAMETER_NAME);
    if (StringUtil.isEmpty(username) || StringUtil.isEmpty(token)) {
      return HttpAuthenticationResult.notApplicable();
    }
    if (!mySuperUserData.getAuthenticationToken().equals(token)){
      sendUauthenticated(request, response, "Wrong authentication token supplied");
      return HttpAuthenticationResult.unauthenticated();
    }
    if (myUserModel.findUserAccount(null,username) == null){
      sendUauthenticated(request, response, "Wrong username supplied: no such user");
      return HttpAuthenticationResult.unauthenticated();
    }
    return HttpAuthenticationResult.authenticated(new ServerPrincipal(null, username));
  }

  private void sendUauthenticated(final HttpServletRequest request, final HttpServletResponse response, final String message) throws IOException {
    HttpAuthUtil.setUnauthenticatedReason(request, message);
    response.sendError(HttpServletResponse.SC_UNAUTHORIZED, message);
  }
}
